
From: Juerg Billeter <juerg@paldo.org>

This patch - based on
http://marc.theaimsgroup.com/?l=linux-kernel&m=110055503031009&w=2 -
makes ACPI S3 wakeup work for me on a ThinkPad T40p laptop with a SMP
kernel. Without it only UP kernels work. I've been using the patch for
three months now without any issues.

The ACPI resume code currently uses a real-mode 16-bit lgdt instruction to
reload the GDT.  This only restores the lower 24 bits of the GDT base
address.  In recent SMP kernels, the GDT seems to have moved out of the
lower 16 megs, thereby causing the ACPI resume to fail -- an invalid GDT
was being loaded.

Signed-off-by: Juerg Billeter <juerg@paldo.org>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/arch/i386/kernel/acpi/wakeup.S |    5 +++--
 1 files changed, 3 insertions(+), 2 deletions(-)

diff -puN arch/i386/kernel/acpi/wakeup.S~acpi-fix-reloading-gdt-on-acpi-s3-wakeup arch/i386/kernel/acpi/wakeup.S
--- 25/arch/i386/kernel/acpi/wakeup.S~acpi-fix-reloading-gdt-on-acpi-s3-wakeup	Fri Apr  8 16:43:55 2005
+++ 25-akpm/arch/i386/kernel/acpi/wakeup.S	Fri Apr  8 16:43:55 2005
@@ -74,8 +74,9 @@ wakeup_code:
 	movw	%ax,%fs
 	movw	$0x0e00 + 'i', %fs:(0x12)
 	
-	# need a gdt
-	lgdt	real_save_gdt - wakeup_code
+	# need a gdt -- use lgdtl to force 32-bit operands, in case
+	# the GDT is located past 16 megabytes
+	lgdtl	real_save_gdt - wakeup_code
 
 	movl	real_save_cr0 - wakeup_code, %eax
 	movl	%eax, %cr0
_
